/**
 * Union College CSc-244 AI Project 3 - Stack >
 * Stack is a class that will implement a stack using an array. It will
 * utilize the typical stack manipulators (e.g., push, pop, etc.)
 * 
 * << Source: CSc-077 Data Structures - RJ Conti, Project 4, 02.13.2002 >> 
 * << Source: "Data Structures in Java" by TA Standish, page 174. >>
 *
 * @author  TJ Bush & RJ Conti
 * @version 03.11.2004
 */

public class Stack
{
    // instance variable(s) ------------------------------------------------
    private int  count;                 // number of items in array
    private int  capacity;              // array length
    private int  capacityIncrement;     // increment amount
    private aNode[] itemArray;          // stack array of aNodes
    // ---------------------------------------------------------------------

    // constructors(s) -----------------------------------------------------
    public Stack()
    {
        // initialize instance variables
        count               =  0;  
        capacity            = 25;      
        capacityIncrement   =  5;
        itemArray           = new aNode[capacity];
    } // end Stack constructor
    // ---------------------------------------------------------------------

    // instance method(s) --------------------------------------------------
    /**
     * size()
     * 
     * Returns the size of the stack.
     * 
     * @param   n/a
     * @return  the size of the stack
     */
    public int size() 
    {
       return count;
    } // end size

    /**
     * isEmpty()
     * 
     * Determines if the stack is empty (true) or not (false).
     * 
     * @param   n/a
     * @return  true if stack is empty, else false
     */
    public boolean isEmpty() 
    {
        if (count == 0)
        {
            return true;
        } // end if
        else
        {
            return false;
        } // end else
    } // end isEmpty

    /**
     * push()
     * 
     * Pushes a value onto the stack. Increases the size of the stack
     * as required.
     * 
     * @param   X      - the value to be pushed onto the stack
     * @return  n/a
     */
    public void push(aNode X) 
    {  
        if (count == capacity) // stack is full, expand it
        {    
            capacity = capacity + capacityIncrement;  
            aNode[] tempArray = new aNode[capacity];    
            for (int i = 0; i < count; i++) 
            {
                tempArray[i] = itemArray[i];
            } // end for
            itemArray = tempArray;
        } // end if     
        itemArray[count++] = X;    
    } // end push

    /**
     * pop()
     * 
     * Pops a value off the stack, decrements the size of the stack, and 
     * returns the value.
     * 
     * @param   n/a
     * @return  the value popped from the stack
     */
    public aNode pop()
    {
        if (count == 0) // stack is empty
        {  
           return null;                           
        } // end if
        else 
        {     
           return itemArray[--count];      
        } // end else
    } // end pop

    /**
     * value()
     * 
     * Returns the node at the specified location in stack.
     * 
     * @param   n/a
     * @return  the node at the specified location
     */
    public aNode value(int location)
    {    
        if (count == 0) // stack empty
        {
            return null;
        } // end if
        else 
        {   
            return itemArray[location];      
        } // end else
    } // end value()     

    /**
     * jumbleStackRandomly()
     * 
     * Returns a randomly rearranged version of the stack.
     * 
     * @param   n/a
     * @return  the node at the specified location
     */
    public Stack jumbleStackRandomly()
    {
        Stack jumble = new Stack();         // stack to hold jumbled stack
        boolean alreadyInThere = false;     // flag to control filling array
        boolean keepLooking    = false;     // flag to look for unique number
        int[] randIndex = new int[count];   // array of random indicies
        int randNo = 0;                     // random number

        //initialize array prior to use
        for (int a = 0; a < count; a++)
        {
            randIndex[a] = -1;  // a value that can't possibly be an array index
        } // end for a

        // generate an array that holds random indicies
        for (int i = 0; i < count; i++)
        {
            keepLooking = true;            

            // keep looking looking until a jumbled array of indicies is found
            while (keepLooking == true)
            {
                randNo = (int)( (1000 * Math.random()) % count);
                alreadyInThere = false;

                // check if number is already in there
                for (int j = 0; j < count; j++)
                {
                    if (randIndex[j] == randNo)
                    {
                        alreadyInThere = true;
                    } //end if
                } // end for j

                if (alreadyInThere == true)
                    keepLooking = true;
                else 
                    keepLooking = false;

            } // end while alreadyInThere
            randIndex[i] = randNo;      // randNo is unique therefore use it
        } // end for i

        // perform the jumble
        for (int s = 0; s < this.count; s++)
        {
            jumble.push(value(randIndex[s]));
        } // end for s
        return jumble;
    } // end jumbleStackRandomly()
    // ---------------------------------------------------------------------
} // end Stack