/**
 * Union College CSc-244 AI Project 3 - aNode >
 * aNode is a class that houses a typical Project 3 node. This node stores
 * the following pertinent information:
 * 
 * 1. board    - an instance of a checker board
 * 2. moves    - the positions to move to get from the initial state to the
 *               board state (i.e., successor state)
 * 3. h_n      - the boards heuristic function
 * 4. numJumps - the number of jumps that the board state encountered from
 *               the initial state
 *
 * @author  TJ Bush & RJ Conti
 * @version 03.11.2004
 */
import java.util.*;     // for Vector
import java.lang.*;     // for Integer

public class aNode
{
    // instance variable(s) ------------------------------------------------
    private char[] board;       // checker board instance
    private Vector moves;       // sequence of moves to get to successor
    private int    h_n;         // heuristic function
    private int    numJumps;    // number of jumps to get to board state
    // ---------------------------------------------------------------------

    // constructors(s) -----------------------------------------------------
    public aNode()
    {
        moves    = new Vector();
        h_n      = 0;
        numJumps = 0;
    } // end aNode constructor
    
    public aNode(char[] b)
    {
        board    = b;
        moves    = new Vector();
        h_n      = 0;
        numJumps = 0;
    } // end aNode constructor
    // ---------------------------------------------------------------------
    
    // instance method(s) --------------------------------------------------
    /**
     * incNumJumps()
     * 
     * This method increments numJumps by 1. This acts as the jump counter.
     * 
     * @param       n/a
     * @return      n/a 
     */
    public void incNumJumps()
    {    
        numJumps++;
    } // end incNumJumps()

    /**
     * getBoard()
     * 
     * This method returns board.
     * 
     * @param       n/a
     * @return      board 
     */
    public char[] getBoard()
    {
        return board;
    } // end getBoard()
 
    /**
     * setBoard()
     * 
     * Given a board, this method replaces board with it.
     * 
     * @param       newBoard - new board
     * @return      n/a 
     */
    public void setBoard(char[] newBoard)
    {
        board = newBoard;
    } // end setBoard()

    /**
     * getMoves()
     * 
     * This method returns moves.
     * 
     * @param       n/a
     * @return      moves 
     */
    public Vector getMoves()
    {
        return moves;
    } // end getMoves()

    /**
     * setMoves()
     * 
     * Given a Vector of moves, this method replaces moves with it.
     * 
     * @param       m   -   the moves
     * @return      n/a 
     */
    public void setMoves(Vector m)   
    {
        moves = m;
    } // end setMoves()

    /**
     * getH()
     * 
     * This method returns h_n.
     * 
     * @param       n/a
     * @return      h_n 
     */
    public int getH()   
    {
        return h_n;
    } // end getH()

    /**
     * setH()
     * 
     * Given a heuristic, this method replaces h_n with it.
     * 
     * @param       h   -   heuristic
     * @return      n/a 
     */
    public void setH(int h)   
    {
        h_n = h;
    } // end getH()

    /**
     * getJumps()
     * 
     * This method returns numJumps.
     * 
     * @param       n/a
     * @return      numJumps
     */
    public int getJumps()   
    {
        return numJumps;
    } // end getJumps()

    /**
     * setJumps()
     * 
     * Given the number of jumps, this method replaces numJumps with it.
     * 
     * @param       j   -   number of jumps
     * @return      n/a 
     */
    public void setJumps(int j)   
    {
        numJumps = j;
    } // end setJumps() 
    
    /**
     * addMove()
     * 
     * This method adds a move to moves.
     * 
     * @param       val  - the move to add to moves
     * @return      n/a 
     */
    public void addMove(int val)
    {    
        moves.addElement(new Integer(val));
    } // end addMove()

    /**
     * copyNode()
     * 
     * This method returns a copy of the node.
     * 
     * @param       n/a
     * @return      a copy of the node
     */
    public aNode copyNode()
    {
        aNode copy = new aNode(getBoard());
        for (int i = 0; i < moves.size(); i++)
        {
            copy.addMove(((Integer)moves.get(i)).intValue());
        }
        copy.h_n = h_n;
        copy.numJumps = numJumps;
        return copy;
    } // end copyNode()

    /**
     * printNode()
     * 
     * This method prints the contents of the node.
     * 
     * @param       n/a
     * @return      n/a 
     */
    public void printNode()
    {    
        System.out.println("Successor State:");
        int piece = 0;  // index into checkerBoard array for printing the 
                        // pieces
        
        System.out.println();
        System.out.println("               RED               ");
        System.out.println("---------------------------------");

        // print the checker board one row at a time
        for (int row= 0; row < 8; row++)
        {
            System.out.print("|");
            if ( (row + 1) % 2 == 1)  // the row is odd
            {
                for (int c = 0; c < 8; c++)
                {
                    if ( (c + 1) % 2 == 1)  // the column is odd
                    {
                        System.out.print(" - |");
                    } // end if odd column
                    else                    // the column is even
                    {
                        System.out.print(" " + board[piece] + " |");
                        piece++;
                    } // end else column even
                } // end for c
            } // end if odd row

            else                    // the row is even
            {
                for (int k = 0; k < 8; k++)
                {
                    if ( (k + 1) % 2 == 1)  // the column is odd
                    {
                        System.out.print(" " + board[piece] + " |");
                        piece++;
                    } // end if
                    else                    // the column is even
                    {
                        System.out.print(" - |");
                    } // end else column even
                } // end for k
            } // end else row even
            System.out.println();
            System.out.println("---------------------------------");
        } // end for row
        System.out.println("              WHITE              ");   
        System.out.println();

        
        System.out.println("h(n) = " + h_n);
        System.out.print("Sequence of moves = ");
        for (int i = 0; i < moves.size(); i++)
        {
            System.out.print("[" + (((Integer)moves.get(i)).intValue()+1)
                                + "] ");
        } // end for i
        System.out.println();
        System.out.print("Number of jumps = " + numJumps);
        System.out.println("\n");
    } // end printNode()
    // ---------------------------------------------------------------------
} // end aNode